/***********************************************************************
 *    DecoEmojiList
 *
 * [File]
 *     DecoEmojiList.java
 *
 *     Copyright (C) 2011-2012 OMRON SOFTWARE Co., Ltd.  All Rights Reserved.
 *
 * [Description]
 *     DecoEmojiList Main Activity Class.
 ************************************************************************/
package jp.co.omronsoft.android.decoemojilist_docomo;

import android.app.Activity;
import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.ServiceConnection;
import android.content.res.Configuration;
import android.content.res.Resources;
import android.graphics.Color;
import android.os.Bundle;
import android.os.Handler;
import android.os.IBinder;
import android.os.Message;
import android.os.RemoteException;
import android.text.Annotation;
import android.text.Spannable;
import android.text.SpannableString;
import android.text.SpannableStringBuilder;
import android.util.Log;
import android.view.GestureDetector;
import android.view.GestureDetector.OnGestureListener;
import android.view.Gravity;
import android.view.LayoutInflater;
import android.view.MotionEvent;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.ViewGroup;
import android.view.WindowManager;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.GridView;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.ScrollView;
import android.widget.Spinner;
import android.widget.TextView;
import android.widget.Toast;

import jp.co.omronsoft.android.decoemojilist_docomo.constants.Const;
import jp.co.omronsoft.android.decoemojilist_docomo.DecoEmojiInfo;
import jp.co.omronsoft.android.decoemojimanager_docomo.interfacedata.DecoEmojiCategoryInfo;
import jp.co.omronsoft.android.decoemojimanager_docomo.interfacedata.DecoEmojiContract;
import jp.co.omronsoft.android.decoemojimanager_docomo.interfacedata.IDecoEmojiConstant;
import jp.co.omronsoft.android.decoemojimanager_docomo.interfacedata.IDecoEmojiManager;
import jp.co.omronsoft.android.emoji.EmojiAssist;

import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Locale;

/**
 * 一覧表示制御アプリ メインアクティビティクラス.
 */
public class DecoEmojiList extends Activity implements Runnable {

    /** For Debug Class Name. */
    private static final String CLASS_NAME = "DecoEmojiList#";

    /** DecoEmojiManagerから取得したカテゴリ情報一覧を表すフィールド. */
    private DecoEmojiCategoryInfo[] mCategoryInfoArray;

    /** 現在表示しているデコ絵文字のURI一覧を表すフィールド. */
    private List<String> mCurrentDecoEmojiUriList;

    /** 現在表示しているカテゴリ情報のインデックスを表すフィールド. */
    private int mCurrentCategoryDataIndex = 0;

    /** 現在表示しているキャリア絵文字カテゴリ情報のインデックスを表すフィールド. */
    private int mCurrentCarrierCategoryDataIndex = 0;

    /** 回転前のカテゴリ情報のインデックスを表すフィールド. */
    private int mCurrentCategoryDataIndexTemp = 0;

    /** 回転前のキャリア絵文字カテゴリ情報のインデックスを表すフィールド. */
    private int mCurrentCarrierCategoryDataIndexTemp = 0;

    /** 現在表示しているカテゴリ情報のページ数を表すフィールド. */
    private int mCurrentCategoryDataPage = Const.CATEGORY_DATA_FIRST_PAGE;

    /** 入力履歴テーブル操作用DAOクラスを表すフィールド. */
    private DecoEmojiListDao mInputHistoryDao;

    /** IMEに返却するリストを表すフィールド. */
    private ArrayList<DecoEmojiListResultInfo> mReturnValue = new ArrayList<DecoEmojiListResultInfo>();

    /** デコ絵文字一覧のスクロールビューを表すフィールド. */
    private ScrollView mBaseScroll;

    /** デコ絵文字一覧をスクロールさせるレイアウトを表すフィールド. */
    private LinearLayout mDecoEmojiListBaseLinear;

    /** デコ絵文字一覧の横を整形するレイアウトを表すフィールド. */
    private LinearLayout mDecoEmojiListInner;

    /** カテゴリ内ページ切替(上)ボタンを表すフィールド. */
    private ImageView mReadMoreButtonBack;

    /** カテゴリ内ページ切替(下)ボタンを表すフィールド. */
    private ImageView mReadMoreButtonNext;

    /** 確定ボタンを表すフィールド. */
    private ImageView mEnterButton;

    /** プレビューテキストを表すフィールド. */
    private TextView mPreviewText;

    /** デコ絵文字タブを表すフィールド. */
    private TextView mSwitchEmojiTabDeco;

    /** キャリア絵文字タブを表すフィールド. */
    private TextView mSwitchEmojiTabCarrier;

    /** カテゴリ名称を表すフィールド. */
    private Spinner mCategoryName;

    /** バックスペースボタンを表すフィールド. */
    private ImageView mBackSpaceButton;

    /** ボタン領域を表すフィールド. */
    private LinearLayout mBaseButtonsLinear;

    /** 入力履歴が存在するかを表すフィールド. */
    private boolean mIsExistInputHistory;

    /** ListViewManagerを表すフィールド. */
    private DecoEmojiListViewManager mListViewManager;

    /** DecoEmojiManagerバインド用オブジェクトを表すフィールド. */
    private IDecoEmojiManager mIDecoEmojiManagerIF = null;

    /** 現在のロケールが日本語かを表すフィールド. */
    private boolean mIsLanguageJapan = false;

    /** デコ／キャリア絵文字切替判定用文字列を表すフィールド. */
    private int mSwitchEmojiFlg;

    /** キャリア絵文字、デコレーション絵文字の表示個数切替用 */
    private int mCurrentInputType;
    private static final int INPUT_TYPE_SINGLE = 1;

    /** 現在表示している絵文字を表すフィールド. */
    private int mCurrentEmojiValue;

    /**
     * IMEの領域を表すフィールド. [0]：縦画面時の縦幅 [1]：横画面時の縦幅
     */
    private int[] mCurrentImeHeight = new int[2];

    /** フリックイベント時のフリック方向を表すフィールド. */
    private int mFlickDirection = Const.FLICK_NONE;

    /** フリックイベント等を検知するジェスチャーを表すフィールド. */
    private GestureDetector mGestureDetector;

    /** フリックされたかを表すフィールド. */
    private boolean mIsFlick = false;

    /** ページスクロール時のスクロール開始地点(Y座標)を表すフィールド. */
    private int mScrollFrom = 0;

    /** ページスクロール時のスクロール終了地点(Y座標)を表すフィールド. */
    private int mScrollTo = 0;

    /** finish()を実行したかを表すフィールド. */
    private boolean isSendFinish = false;

    /** 絵文字描画用インスタンスを表すフィールド. */
    private EmojiAssist mEmojiAssist;
    
    /** 一覧に表示される絵文字のサイズ(sp) */
    private static final float DECOEMOJI_SIZE = 20.0f;

    /** キャリア絵文字カテゴリリスト*/
    private List<CarrierEmojiCategoryInfo> mCarrierEmojiCategoryInfoList;
    
    /** キャリア絵文字入力履歴が存在するかを表すフィールド */
    private boolean mIsExistCarrierInputHistory;

    /** キャリア絵文字リソースID配列 */
    private int[] mCarrierEmojiResourceIds = {
        R.array.emoji_docomo_item,
    };

    /** 絵文字Dリソース読み込みリトライ */
    private static final int RETRY_TIME = 5000;
    private static final int MAX_RETRY_COUNT = 6;
    private int mRetryCount = MAX_RETRY_COUNT;

    /** 絵文字Dリソースチェック */
    private int NO_PICT_D_DATA_FLAG = 1;
    private int mNoPictDData = 0;
    private boolean mCategoryInfoCheck = false;

    /** 絵文字Dリソース読み込みリトライ用スレッド */
    private Thread retryThread;

    /** 絵文字種別 */
    private int mEmojiType = 0x0000;

    /** ページの絵文字表示情報 */
    private LinkedHashMap<Integer,PageInfo> mPageInfo = new LinkedHashMap<Integer,PageInfo>();

    /** 最大ページを表すフィールド */
    private int mMaxPage = 0;

   /** View's. */
    private ArrayList<TextView> mAddViewList;

    /** @see android.app.Activity#onCreate */
    @Override
    public void onCreate(Bundle b) {
        if (Const.DEBUG) {
            Log.d(Const.TAG, CLASS_NAME + "onCreate() start.");
        }

        super.onCreate(b);
        
        mAddViewList = new ArrayList<TextView>();
        mAddViewList.clear();

        // 入力履歴テーブル操作用DAOクラスを生成する
        mInputHistoryDao = new DecoEmojiListDao(getApplicationContext());
        // キャリア絵文字一覧を取得する
        loadCarrierEmojiResources();

        // 絵文字描画用インスタンスを取得する
        mEmojiAssist = EmojiAssist.getInstance();
        mEmojiAssist.setPictureScale(true);

        Intent intentFromIme = (Intent) getIntent();

        // デコ／キャリア絵文字タブ切替のフラグを設定する
        mSwitchEmojiFlg = intentFromIme.getIntExtra(Const.EXTRA_SWITCH_EMOJI_FLG,
                Const.SWITCH_VALUE_BOTH);
        if (mSwitchEmojiFlg != Const.SWITCH_VALUE_DECO
                && mSwitchEmojiFlg != Const.SWITCH_VALUE_CARRIER) {
            // デコ／キャリア絵文字タブ
            mSwitchEmojiFlg = Const.SWITCH_VALUE_BOTH;
            mCurrentEmojiValue = Const.SWITCH_VALUE_DECO;
        } else {
            mCurrentEmojiValue = mSwitchEmojiFlg;
        }

        mEmojiType = intentFromIme.getIntExtra(IDecoEmojiConstant.EMOJI_TYPE, mEmojiAssist.EMOJITYPE_PICTD_SQ20);

        // 起動元のIMEの縦幅を設定する
        mCurrentImeHeight[0] = intentFromIme.getIntExtra(Const.EXTRA_CURRENT_IME_HEIGHT_PORT,
                0);
        mCurrentImeHeight[1] = intentFromIme.getIntExtra(Const.EXTRA_CURRENT_IME_HEIGHT_LAND,
                0);

        mCurrentInputType = intentFromIme.getIntExtra(Const.EXTRA_CURRENT_INPUT_TYPE, 0);

        // ロケールを設定する
        mIsLanguageJapan = Locale.JAPAN.equals(Locale.getDefault());

        // 画面に表示する情報をコンテントに登録する
        mListViewManager = new DecoEmojiListViewManager();
        WindowManager wm = (WindowManager) getSystemService(Context.WINDOW_SERVICE);
        int width = wm.getDefaultDisplay().getWidth();
        int height = wm.getDefaultDisplay().getHeight();
        Resources res = getResources();
        LayoutInflater inflater = getLayoutInflater();
        ViewGroup viewGroup = mListViewManager.initView(res, inflater, width, height,
                mCurrentImeHeight, mCurrentInputType);
        setContentView(viewGroup);

        // 画面に配置するコンポーネントの設定を行う
        setComponentEvents();

        // デコ／キャリア絵文字タブの設定を行う
        setSwitchEmojiTabEvent();

        // カテゴリ名称を設定する
        setCategoryName();

        // サービス(DecoEmojiManager)をバインドする
        Intent intentBindSearvice = new Intent(IDecoEmojiManager.class.getName());
        bindService(intentBindSearvice, serviceConn, Context.BIND_AUTO_CREATE);

        // フリックイベント等のジェスチャーを設定する
        mGestureDetector = new GestureDetector(getApplicationContext(),
                new OnGestureListener() {
                    @Override
                    public boolean onDown(MotionEvent e) {
                        return false;
                    }

                    @Override
                    public boolean onFling(MotionEvent e1, MotionEvent e2,
                            float velocityX, float velocityY) {
                        if (velocityY > 0) {
                            // previous
                            mFlickDirection = Const.FLICK_PREVIOUS;
                        } else {
                            // next
                            mFlickDirection = Const.FLICK_NEXT;
                        }
                        return true;
                    }

                    @Override
                    public void onLongPress(MotionEvent e) {
                    }

                    @Override
                    public boolean onScroll(MotionEvent e1, MotionEvent e2,
                            float distanceX, float distanceY) {
                        return false;
                    }

                    @Override
                    public void onShowPress(MotionEvent e) {
                    }

                    @Override
                    public boolean onSingleTapUp(MotionEvent e) {
                        return false;
                    }
                });

        if (Const.DEBUG) {
            Log.d(Const.TAG, CLASS_NAME + "onCreate() end.");
        }
    }

    /**
     * キャリア絵文字のカテゴリ・文字コードをリソースから取得する
     */
    private void loadCarrierEmojiResources() {
        
        mCarrierEmojiCategoryInfoList = new ArrayList<CarrierEmojiCategoryInfo>();
        
        for (int rid : mCarrierEmojiResourceIds) {
            String[] carrierEmojiArry = getResources().getStringArray(rid);
            
            for (String item : carrierEmojiArry) {
                
                if (isCategory(item)) {
                    CarrierEmojiCategoryInfo info = new CarrierEmojiCategoryInfo();
                    info.setCategoryName(item.replaceAll("\\[|\\]", ""));
                    mCarrierEmojiCategoryInfoList.add(info);
                } else {
                    mCarrierEmojiCategoryInfoList.get(mCarrierEmojiCategoryInfoList.size() - 1).getmEmojiCodeList().add(item);
                }
        
             }
        }
        
        // 入力履歴の存在チェック
        mIsExistCarrierInputHistory = mInputHistoryDao.isExistHistoriesCarrier();
        
        // 入力履歴が存在する場合
        if (mIsExistCarrierInputHistory) {
            // カテゴリの1件目を入力履歴から設定する
            CarrierEmojiCategoryInfo historyInfo = new CarrierEmojiCategoryInfo();
            historyInfo.setCategoryName(getResources().getString(R.string.input_history_name));
            mCarrierEmojiCategoryInfoList.add(0, historyInfo);
            
        }
    }

    /**
     * 文字列がカテゴリかどうかを判定する
     * @param word
     * @return
     */
    private boolean isCategory(String word) {
        int length = word.length();
        return ((length > 3) && (word.charAt(0) == '['));
    }

    /** @see android.app.Activity#onDestroy */
    @Override
    protected void onDestroy() {
        if (Const.DEBUG) {
            Log.d(Const.TAG, CLASS_NAME + "onDestory() start.");
        }

        super.onDestroy();
        if (mSwitchEmojiFlg == Const.SWITCH_VALUE_BOTH
                || mCurrentEmojiValue == Const.SWITCH_VALUE_DECO) {
            // デコ絵文字
            if (mIDecoEmojiManagerIF != null) {
                unbindService(serviceConn);
            }
        }

        // EmojiAssistに登録されたTextViewをクリアする
        mEmojiAssist.clearView();
        mAddViewList.clear();

        if (Const.DEBUG) {
            Log.d(Const.TAG, CLASS_NAME + "onDestory() end.");
        }
    }

    /** @see android.app.Activity#onStop */
    @Override
    protected void onStop() {
        if (Const.DEBUG) {
            Log.d(Const.TAG, CLASS_NAME + "onStop() start.");
        }

        super.onStop();
        sendResultData(false);

        if (Const.DEBUG) {
            Log.d(Const.TAG, CLASS_NAME + "onStop() end.");
        }
    }

    /** @see android.app.Activity#onResume */
    @Override
    protected void onResume() {
        if (Const.DEBUG) {
            Log.d(Const.TAG, CLASS_NAME + "onResume() start.");
        }

        super.onResume();
        Boolean isDeco = false;
        if (mAddViewList != null && mAddViewList.size() > 0) {
            for (int i = 0; mAddViewList.size() > i ; i++) {
                Spannable text = new SpannableString(mAddViewList.get(i).getText());
                int ret = mEmojiAssist.checkTextData(text);
                if (ret > EmojiAssist.TYPE_TEXT_EMOJI) {
                    isDeco = true;
                    break;
                    
                }
            }
            if (isDeco) {
                mEmojiAssist.startAnimation();
            } else {
                mEmojiAssist.stopAnimation();
            }
        } else {
            mEmojiAssist.stopAnimation();
        }

        if (Const.DEBUG) {
            Log.d(Const.TAG, CLASS_NAME + "onResume() end.");
        }
    }

    /** @see android.app.Activity#onPause */
    @Override
    protected void onPause() {
        if (Const.DEBUG) {
            Log.d(Const.TAG, CLASS_NAME + "onPause() start.");
        }

        super.onPause();
        mEmojiAssist.stopAnimation();

        if (Const.DEBUG) {
            Log.d(Const.TAG, CLASS_NAME + "onPause() end.");
        }
    }

    /** @see android.app.Activity#onConfigurationChanged */
    @Override
    public void onConfigurationChanged(Configuration newConfig) {
        if (Const.DEBUG) {
            Log.d(Const.TAG, CLASS_NAME + "onConfigurationChanged() start.");
        }

        super.onConfigurationChanged(newConfig);

        // 回転後の情報をコンテントに登録する
        WindowManager wm = (WindowManager) getSystemService(Context.WINDOW_SERVICE);
        int width = wm.getDefaultDisplay().getWidth();
        int height = wm.getDefaultDisplay().getHeight();
        Resources res = getResources();
        LayoutInflater inflater = getLayoutInflater();
        ViewGroup viewGroup = mListViewManager.initView(res, inflater, width, height,
                mCurrentImeHeight, mCurrentInputType);
        setContentView(viewGroup);

        // EmojiAssistに登録されたTextViewをクリアする
        mEmojiAssist.clearView();
        mAddViewList.clear();

        // 画面に配置するコンポーネントの設定を行う
        setComponentEvents();

        // デコ／キャリア絵文字タブの設定を行う
        setSwitchEmojiTabEvent();

        // カテゴリ名称を設定する
        setCategoryName();

        if (mCurrentEmojiValue == Const.SWITCH_VALUE_DECO) {

            // デコ絵文字
            
            // 回転前のカテゴリ情報のインデックスを記憶する
            mCurrentCategoryDataIndexTemp = mCurrentCategoryDataIndex;

            // スクロールバーを初期化する
            mScrollFrom = 0;
            mScrollTo = 0;

            // カテゴリ情報一覧を更新する
            initCategoryDataList();
        } else {
            // キャリア絵文字
            
            // 回転前のカテゴリ情報のインデックスを記憶する
            mCurrentCarrierCategoryDataIndexTemp = mCurrentCarrierCategoryDataIndex;

            // スクロールバーを初期化する
            mScrollFrom = 0;
            mScrollTo = 0;

            // カテゴリ情報一覧を更新する
            initCarrierCategoryDataList();
            
        }

        if (Const.DEBUG) {
            Log.d(Const.TAG, CLASS_NAME + "onConfigurationChanged() end.");
        }
    };

    /**
     * サービス接続の確立／切断用メソッドをオーバーライドする.
     */
    ServiceConnection serviceConn = new ServiceConnection() {

        // サービスとの接続が確立された時に実行される
        public void onServiceConnected(ComponentName componentName, IBinder iBinder) {
            if (Const.DEBUG) {
                Log.d(Const.TAG, CLASS_NAME + "onServiceConnected() start.");
            }

            mIDecoEmojiManagerIF = IDecoEmojiManager.Stub.asInterface(iBinder);

            if (mCurrentEmojiValue == Const.SWITCH_VALUE_DECO) {
                // デコ絵文字を表示する
                initCategoryDataList();
                // カテゴリ情報がない場合はリトライ
                if (mCategoryInfoArray == null || mCategoryInfoCheck == true) {
                    mCategoryInfoCheck = false;
                    if (mNoPictDData != NO_PICT_D_DATA_FLAG) {
                        retryCheck();
                    }
                }
            } else {
                // キャリア絵文字を表示する
                initCarrierCategoryDataList();
            }

            if (Const.DEBUG) {
                Log.d(Const.TAG, CLASS_NAME + "onServiceConnected() end.");
            }
        }

        // サービスとの接続が切断された時に実行される
        public void onServiceDisconnected(ComponentName componentName) {
            mIDecoEmojiManagerIF = null;
        }
    };

    /**
     * カテゴリ情報一覧を更新する処理を行う.
     */
    private void initCategoryDataList() {
        List<DecoEmojiCategoryInfo> categoryInfoList = new ArrayList<DecoEmojiCategoryInfo>();

        // カテゴリ情報一覧の要素数計算用
        int categoryIdNameListSize = 0;
        
        mPageInfo = new LinkedHashMap<Integer,PageInfo>();
        
        categoryInfoList = mInputHistoryDao.getCategoryInfoList(mListViewManager, mEmojiType);

        categoryIdNameListSize += categoryInfoList.size();
        
        // 入力履歴の初期処理を行う
        if (!isFinishing()) {
            mInputHistoryDao.init(mListViewManager.getDecoEmojiMaxDrawableCount(), mEmojiType);
            mIsExistInputHistory = mInputHistoryDao.isExistHistories();
            if (mIsExistInputHistory) {
                categoryIdNameListSize++;
            }
        }

        // カテゴリ情報を取得する
        if (categoryIdNameListSize != 0) {

            if (Const.DEBUG) {
                Log.d(Const.TAG, CLASS_NAME + "initCategoryDataList() categoryIdNameListSize["
                        + categoryIdNameListSize + "]");
            }

            mCategoryInfoArray = new DecoEmojiCategoryInfo[categoryIdNameListSize];
            for (int i = 0; i < categoryIdNameListSize; i++) {
                if (mIsExistInputHistory) {
                    if (i == 0) {
                        // 入力履歴が存在する時、1件目をダミー情報として保存する
                        mCategoryInfoArray[0] = null;
                        mCategoryInfoCheck = true;
                    } else {
                        mCategoryInfoCheck = false;
                        // カテゴリ情報を保存する
                        if (categoryInfoList.get(i - 1) != null) {
                            mCategoryInfoArray[i] = categoryInfoList.get(i - 1);
                        }
                    }
                } else {
                    // カテゴリ情報を保存する
                    if (categoryInfoList.get(i) != null) {
                        mCategoryInfoArray[i] = categoryInfoList.get(i);
                    }
                }
            }
            // 回転前のカテゴリ情報のインデックスを設定する(初回は先頭ページを設定)
            mCurrentCategoryDataIndex = mCurrentCategoryDataIndexTemp;
        } else {
            mCategoryInfoArray = null;
        }

        // カレントページに初期ページを設定する
        mCurrentCategoryDataPage = Const.CATEGORY_DATA_FIRST_PAGE;

        // カテゴリ名称を設定する
        setCategoryName();

        // カテゴリ情報一覧を設定する
        if (mCurrentEmojiValue == Const.SWITCH_VALUE_DECO) {
            toCurrentPage();
        }
    }

    /**
     * リトライ回数チェック処理
     */
    private void retryCheck(){
        // リトライ回数＝6回
        if (mRetryCount != 0) {
            if (!isFinishing()) {
                retryThread = new Thread(this);
                retryThread.start();
            }
        }
        else {
            mNoPictDData = NO_PICT_D_DATA_FLAG;
            mRetryCount = MAX_RETRY_COUNT;
            Toast.makeText(getApplicationContext(), R.string.pict_d_result_message, Toast.LENGTH_SHORT).show();
        }
    }

    /**
     * リトライ待ちウェイト処理
     */
    public void run() {
        try {
            Thread.sleep(RETRY_TIME);
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
        handler.sendEmptyMessage(0);
    }
     
    /**
     * ウェイト完了後処理
     */
    private Handler handler = new Handler() {
        public void handleMessage(Message msg){
            mRetryCount--;
            initCategoryDataList();
            if (mCategoryInfoArray == null || mCategoryInfoCheck == true) {
                mCategoryInfoCheck = false;
                if (mNoPictDData != NO_PICT_D_DATA_FLAG) {
                    retryCheck();
                }
            }
            else {
                mRetryCount = MAX_RETRY_COUNT;
            }
        }
    };

    /**
     * キャリア絵文字カテゴリ情報一覧を更新する処理を行う.
     */
    private void initCarrierCategoryDataList() {

        // 回転前のカテゴリ情報のインデックスを設定する(初回は先頭ページを設定)
        mCurrentCarrierCategoryDataIndex = mCurrentCarrierCategoryDataIndexTemp;

          // カテゴリ名称を設定する
        setCategoryName();

        // カテゴリ情報一覧を設定する
        if (mCarrierEmojiCategoryInfoList != null) {
            showCarrierEmoji(mCurrentCarrierCategoryDataIndex);
        }
        
    }
    
    /**
     * DecoEmojiManagerからデコ絵文字一覧を取得し、画面に表示を行う.
     *
     * @param targetPage
     *            表示するページ数
     */
    private void showCategoryList(int targetPage) {
    	
    	PageInfo pageInfo;
    	
        // カテゴリ情報一覧が無い時、処理をしない
        if (mCategoryInfoArray == null) {
            return;
        }
        	
        if (mPageInfo.containsKey(targetPage)) {
        	pageInfo = mPageInfo.get(targetPage);
        } else {
        	pageInfo = new PageInfo(0, 0, 0, 0);
        }
        
        // デコ絵文字一覧を表示するレイアウトを設定する
        LinearLayout targetLinear = new LinearLayout(getApplicationContext());
        targetLinear.setVisibility(View.GONE);
        targetLinear.setOrientation(LinearLayout.VERTICAL);
        targetLinear.setMinimumHeight(mListViewManager.getDecoEmojiLinearHeight());
        targetLinear.removeAllViews();

        // 表示するデコ絵文字一覧を取得する
        if (mIsExistInputHistory && mCurrentCategoryDataIndex == 0) {
            mCurrentDecoEmojiUriList = null;
            // 保持しているページ数を超えていない時に取得を行う
            if (Const.CATEGORY_DATA_FIRST_PAGE <= targetPage
                    && targetPage <= mMaxPage) {
                mCurrentDecoEmojiUriList = mInputHistoryDao.loadHistories(targetPage, pageInfo.getEmojiStart(), mEmojiType);
            }
            if (mCurrentDecoEmojiUriList != null) {
            	showDecoEmojiList(targetLinear, pageInfo);
            	PageInfo nextPageInfo = new PageInfo(0, pageInfo.getEmojiCount() + pageInfo.getPictureCount(), 
                        0, 0);
            	mPageInfo.put(targetPage + 1, nextPageInfo);
                if (mMaxPage < (targetPage + 1)) {
                    mMaxPage = targetPage + 1;
                }
            }
        } else {
            // DecoEmojiManagerから取得する
            DecoEmojiCategoryInfo categoryInfo = mCategoryInfoArray[mCurrentCategoryDataIndex];
            mCurrentDecoEmojiUriList = new ArrayList<String>();
            
            // Pictureのスタート0の場合、デコ絵文字を取得
            if (pageInfo.getPictureStart() == 0) {
                mCurrentDecoEmojiUriList = mInputHistoryDao
                    .getDecoEmojiUriList(mListViewManager, mEmojiType, targetPage, categoryInfo, pageInfo.getEmojiStart(),false);
            }
            
            // Pictureを取得する。
            if (mListViewManager.getDecoEmojiMaxDrawableCount() > mCurrentDecoEmojiUriList.size() || pageInfo.getEmojiStart() == 0) {
                List<String> currentPictureUriList = mInputHistoryDao
                        .getDecoEmojiUriList(mListViewManager, mEmojiType, targetPage, categoryInfo, pageInfo.getPictureStart(),true);
                mCurrentDecoEmojiUriList.addAll(currentPictureUriList);
            }
            
            if (mCurrentDecoEmojiUriList.size() != 0) {
            	showDecoEmojiList(targetLinear, pageInfo);
            	PageInfo nextPageInfo = new PageInfo(0, pageInfo.getEmojiCount(), 
            			                             0, pageInfo.getPictureCount());
                mPageInfo.put(targetPage + 1, nextPageInfo);
                if (mMaxPage < (targetPage + 1)) {
                    mMaxPage = targetPage + 1;
                }
            }
        }
        mDecoEmojiListBaseLinear.addView(targetLinear);
    }

    
    /**
     * デコ絵文字一覧を表示する。
     * @param targetLinear デコ絵文字を表示するレイアウト
     * @param pageInfo レイアウトのページ情報
     */
    private void showDecoEmojiList(LinearLayout targetLinear, PageInfo pageInfo ){

        if (Const.DEBUG) {
            Log.d(Const.TAG, CLASS_NAME
                    + "showCategoryList() mCurrentDecoEmojiUriList.size["
                    + mCurrentDecoEmojiUriList.size() + "]");
        }

        if (mCurrentDecoEmojiUriList == null || mCurrentDecoEmojiUriList.size() == 0) {
        	return;        	
        }
        
        int decoEmojiCounter = 0;
        int decoEmojiCount = 0;
        int decoPictureCount = 0;
        int decoEmojiWidth;
        int decoEmojiHeight;
        TextView decoEmojiView;
        int viewCnt;
        int preEmojiPosition;

        // 画像の縦横を取得
        int imageHeight = Const.DEFAULT_HEIGHT;
        int imageWidth = Const.DEFAULT_WIDTH;
        int kind = Const.DEFAULT_KIND;
        boolean changeRowFlag = false;

        // 全体のデコ絵文字を表示する(行)
        for (int i = 0; i < mListViewManager.getDecoEmojiMaxRows(); i++) {
            mDecoEmojiListInner = new LinearLayout(getApplicationContext());
            mDecoEmojiListInner.setOrientation(LinearLayout.HORIZONTAL);
            viewCnt = 0;
            preEmojiPosition = 0;
            // 1行分のデコ絵文字を表示する(列)
            for (int j = 0; j < mListViewManager.getDecoEmojiMaxCols()
                    && decoEmojiCounter < mCurrentDecoEmojiUriList.size(); j++) {
                CharSequence charSequence = mEmojiAssist.getDecoEmojiText(Const.PREFIX_URI
                        + mCurrentDecoEmojiUriList.get(decoEmojiCounter).toString(), getApplicationContext());
                decoEmojiView = new TextView(getApplicationContext());
                decoEmojiView.setTextSize(DECOEMOJI_SIZE);
                float textSize = decoEmojiView.getTextSize();
                float defaultTextSize = new TextView(getApplicationContext()).getTextSize();
                SpannableStringBuilder text = (SpannableStringBuilder)charSequence;
                for (Annotation s : text.getSpans(0, 1, Annotation.class)) {
                    if (s.getValue().indexOf(EmojiAssist.SPLIT_KEY) < 0) {
                        continue;
                    }
                    String[] split = s.getValue().split(EmojiAssist.SPLIT_KEY,0);
                    imageWidth = Integer.parseInt(split[1]);
                    imageHeight = Integer.parseInt(split[2]);
                    kind = Integer.parseInt(split[3]);
                }
                if (imageHeight > Const.DEFAULT_HEIGHT ) {
                    int resizeHeight = Const.DEFAULT_HEIGHT;
                    int resizeWidth = imageWidth / imageHeight * Const.DEFAULT_HEIGHT;
                    if (resizeWidth == 0) {
                        resizeWidth = Const.DEFAULT_WIDTH;
                    }
                    imageHeight = resizeHeight;
                    imageWidth = resizeWidth;
                }

                imageHeight = (int)((float)imageHeight * textSize / Const.DEFAULT_WIDTH);
                imageWidth = (int)((float)imageWidth * textSize / Const.DEFAULT_WIDTH) + Const.TEXT_PADDING_LEFT + Const.TEXT_PADDING_RIGHT;

                // 1画面の最終デコ絵文字は縦幅を変更する
                if ((i + 1) == mListViewManager.getDecoEmojiMaxRows()) {
                    decoEmojiHeight = mListViewManager.getDecoEmojiHeightEndOfLine();
                } else {
                    decoEmojiHeight = mListViewManager.getDecoEmojiHeight();
                }

                int length = (int) Math.floor(imageWidth / mListViewManager.getDecoEmojiWidth());
                if (imageWidth % mListViewManager.getDecoEmojiWidth() > 0) {
                    length++;
                }

                if (length >= mListViewManager.getDecoEmojiMaxCols()) {
                    length = mListViewManager.getDecoEmojiMaxCols();
                }

                // デコメピクチャが来た場合は改行する
                if (mCategoryInfoArray[mCurrentCategoryDataIndex] != null
                        && kind == DecoEmojiContract.KIND_PICTURE
                        && !changeRowFlag) {
                    changeRowFlag = true;
                    if (viewCnt > 0) {
                        //改行処理
                        updatePreViewLayOut(viewCnt, decoEmojiCounter, preEmojiPosition, decoEmojiHeight);
                        break;
                    }
                }

                // 絵文字が全体の行の長さを超える場合は改行する
                if (j + length > mListViewManager.getDecoEmojiMaxCols()) {
                    if (viewCnt > 0) {
                        //改行処理
                        updatePreViewLayOut(viewCnt, decoEmojiCounter, preEmojiPosition, decoEmojiHeight);
                        break;
                    }
                }

                // 開始位置移動
                preEmojiPosition = j;
                j = j + length - 1;
                decoEmojiWidth = length * mListViewManager.getDecoEmojiWidth();

                if (j == mListViewManager.getDecoEmojiMaxCols()) {
                    decoEmojiWidth = decoEmojiWidth + (mListViewManager.getDisplayWidth() % mListViewManager.getDecoEmojiMaxCols());
                }
                decoEmojiView = getCategoryTextView(decoEmojiView, decoEmojiCounter,
                        decoEmojiWidth, decoEmojiHeight, charSequence);
                mDecoEmojiListInner.addView(decoEmojiView);
                decoEmojiCounter++;
                viewCnt++;
                
                if (kind !=  DecoEmojiContract.KIND_PICTURE) {
                	decoEmojiCount++;
                } else {
                	decoPictureCount++;
                }
                
            }
            targetLinear.addView(mDecoEmojiListInner);
        }
        targetLinear.setVisibility(View.VISIBLE);
        
        pageInfo.setEmojiCount(decoEmojiCount + pageInfo.getEmojiStart());
        pageInfo.setPictureCount(decoPictureCount + pageInfo.getPictureStart());

    }
    
    
    /**
     * 画面のボタンのイベント設定、カテゴリ名称設定、プレビューテキスト設定を行う.
     */
    private void setComponentEvents() {
        // コンポーネントを定義する
        setComponentFindViewById();

        // プレビューテキスト
        setPreviewText();

        // 確定ボタン
        setEnterButtonEvent();

        // スクロールバー
        setBaseScrollEvent();

        // カテゴリ内ページ切替(上)ボタン
        setReadMoreButtonBackEvent();

        // カテゴリ内ページ切替(下)ボタン
        setReadMoreButtonNextEvent();

        // バックスペースボタン
        setBackSpaceButtonEvent();
    }

    /**
     * コンポーネントを定義する.
     */
    private void setComponentFindViewById() {
        mSwitchEmojiTabDeco = (TextView) findViewById(R.id.switch_emoji_tab_deco_text);
        mSwitchEmojiTabCarrier = (TextView) findViewById(R.id.switch_emoji_tab_carrier_text);
        mPreviewText = (TextView) findViewById(R.id.preview_text);
        mPreviewText.setTextSize(DECOEMOJI_SIZE);
        // 1個確定モードの場合はプレビューテキストエリアを表示しない
        if (mCurrentInputType == INPUT_TYPE_SINGLE) {
            mPreviewText.setVisibility(View.GONE);
        }
        mBaseScroll = (ScrollView) findViewById(R.id.base_scroll);
        mDecoEmojiListBaseLinear = (LinearLayout) findViewById(R.id.base_list_linear);
        mEnterButton = (ImageView) findViewById(R.id.enter_button);
        mReadMoreButtonBack = (ImageView) findViewById(R.id.read_more_back_button);
        mReadMoreButtonNext = (ImageView) findViewById(R.id.read_more_next_button);
        mBackSpaceButton = (ImageView) findViewById(R.id.back_space_button);

        // 1個確定モードの場合はボタンエリア、及びボタンを表示しない
        if (mCurrentInputType == INPUT_TYPE_SINGLE) {
            mBaseButtonsLinear = (LinearLayout) findViewById(R.id.base_buttons_linear);
            mBaseButtonsLinear.setVisibility(View.GONE);
        }

        mCategoryName = (Spinner) findViewById(R.id.category_name_spinner);
    }

    /**
     * IMEに返却する文字列を画面上部のプレビューテキストに設定する.
     */
    private void setPreviewText() {
        if (mReturnValue.size() != 0) {
            mPreviewText.setText("");
            DecoEmojiListResultInfo decoEmojiListResultInfo;
            for (int i = 0; i < mReturnValue.size(); i++) {
                decoEmojiListResultInfo = mReturnValue.get(i);
                
                CharSequence charSequence;
                switch (decoEmojiListResultInfo.getTag()) {
                case Const.SWITCH_VALUE_DECO:
                    // デコ絵文字
                    
                    charSequence = mEmojiAssist.getDecoEmojiText(Const.PREFIX_URI
                            + decoEmojiListResultInfo.getUri(), getApplicationContext());
                    break;

                case Const.SWITCH_VALUE_CARRIER:
                    // キャリア絵文字
                    
                    charSequence = decoEmojiListResultInfo.getCode();
                    
                    break;
                    
                default:
                    
                    throw new IllegalArgumentException("Unsupported tag " + decoEmojiListResultInfo.getTag());
                    
                }

                mPreviewText.append(charSequence);
            }
            addView(mPreviewText);
        }
    }

    /**
     * デコ／キャリア絵文字タブのイベント設定を行う.
     */
    private void setSwitchEmojiTabEvent() {
        if (mSwitchEmojiFlg == Const.SWITCH_VALUE_BOTH) {
            // デコ絵文字タブ：有効 キャリア絵文字タブ：有効
            mSwitchEmojiTabDeco.setBackgroundResource(R.drawable.switch_emoji_tab);
            mSwitchEmojiTabDeco.setMinimumWidth((mListViewManager.getDisplayWidth() / 2));
            mSwitchEmojiTabDeco.setOnClickListener(mSwitchEmojiTabOnClick);
            mSwitchEmojiTabDeco.setTextColor(Color.WHITE);
            mSwitchEmojiTabDeco.setVisibility(View.VISIBLE);
            mSwitchEmojiTabCarrier
                    .setBackgroundResource(R.drawable.switch_emoji_tab_no_select);
            mSwitchEmojiTabCarrier.setMinimumWidth((mListViewManager.getDisplayWidth() / 2));
            mSwitchEmojiTabCarrier.setOnClickListener(mSwitchEmojiTabOnClick);
            mSwitchEmojiTabCarrier.setTextColor(Color.GRAY);
            mSwitchEmojiTabCarrier.setVisibility(View.VISIBLE);
            if (mCurrentEmojiValue == Const.SWITCH_VALUE_CARRIER) {
                // デコ絵文字タブを非活性にする
                mSwitchEmojiTabDeco
                        .setBackgroundResource(R.drawable.switch_emoji_tab_no_select);
                mSwitchEmojiTabDeco.setTextColor(Color.GRAY);
                // キャリア絵文字タブを活性にする
                mSwitchEmojiTabCarrier.setBackgroundResource(R.drawable.switch_emoji_tab);
                mSwitchEmojiTabCarrier.setTextColor(Color.WHITE);
            }
        } else if (mSwitchEmojiFlg == Const.SWITCH_VALUE_DECO) {
            // デコ絵文字タブ：有効 キャリア絵文字タブ：無効
            mSwitchEmojiTabDeco.setBackgroundResource(R.drawable.switch_emoji_tab);
            mSwitchEmojiTabDeco.setMinimumWidth(mListViewManager.getDisplayWidth());
            mSwitchEmojiTabDeco.setTextColor(Color.WHITE);
            mSwitchEmojiTabDeco.setVisibility(View.VISIBLE);
            mSwitchEmojiTabCarrier.setVisibility(View.GONE);
        } else if (mSwitchEmojiFlg == Const.SWITCH_VALUE_CARRIER) {
            // デコ絵文字タブ：無効 キャリア絵文字タブ：有効
            mSwitchEmojiTabDeco.setVisibility(View.GONE);
            mSwitchEmojiTabCarrier.setBackgroundResource(R.drawable.switch_emoji_tab);
            mSwitchEmojiTabCarrier.setMinimumWidth(mListViewManager.getDisplayWidth());
            mSwitchEmojiTabCarrier.setTextColor(Color.WHITE);
            mSwitchEmojiTabCarrier.setVisibility(View.VISIBLE);
        }
    }

    /**
     * 確定ボタンのイベント設定を行う.
     */
    private void setEnterButtonEvent() {
        mEnterButton.setOnClickListener(new View.OnClickListener() {
            /**
             * 確定ボタン押下時のイベント
             */
            public void onClick(View view) {

                // 入力履歴をデータベースに保存する
                if (mReturnValue.size() != 0) {
                    DecoEmojiListResultInfo decoEmojiListResultInfo;

                    for (int i = 0; i < mReturnValue.size(); i++) {
                        decoEmojiListResultInfo = mReturnValue.get(i);
                        
                        switch (decoEmojiListResultInfo.getTag()) {
                        // デコ絵文字
                        case Const.SWITCH_VALUE_DECO:
                            
                            try {
                                mIDecoEmojiManagerIF.aidl_changeHistory(decoEmojiListResultInfo.getUri());
                            } catch (RemoteException e) {
                                e.printStackTrace();
                            }
                            break;

                        // キャリア絵文字
                        case Const.SWITCH_VALUE_CARRIER:
                            
                             mInputHistoryDao.updateHistoryCarrier(decoEmojiListResultInfo.getCode());
                            
                            break;
                            
                        default:
                            
                            throw new IllegalArgumentException("Unsupported tag " + decoEmojiListResultInfo.getTag());
                            
                        }
                        
                    }
                }

                if (Const.DEBUG) {
                    Log.d(Const.TAG, CLASS_NAME
                            + "setEnterButtonEvent.onClick() mReturnValue.size["
                            + mReturnValue.size() + "]");
                }

                // 結果を返却する
                sendResultData(true);
            }
        });
    }

    /**
     * スクロールバーのイベント設定を行う.
     */
    private void setBaseScrollEvent() {
        // スクロールバー(Y方向)を不可視にする
        if (mCurrentEmojiValue == Const.SWITCH_VALUE_DECO) {
            mBaseScroll.setVerticalScrollBarEnabled(false);
        }

        mBaseScroll.setOnTouchListener(new View.OnTouchListener() {
            @Override
            public boolean onTouch(View v, MotionEvent event) {

              if (mCurrentEmojiValue == Const.SWITCH_VALUE_DECO) {
                  // デコ絵文字
                  
                // カテゴリ情報一覧が無い時、処理をしない
                if (mCategoryInfoArray == null) {
                    return false;
                }

                // フリック(orフリング)中は処理しない
                if (mIsFlick == false) {
                    mIsFlick = mGestureDetector.onTouchEvent(event);
                }

                switch (event.getAction()) {
                    case MotionEvent.ACTION_DOWN:
                        mIsFlick = false;
                        break;
                    case MotionEvent.ACTION_MOVE:
                        mIsFlick = false;
                        break;
                    case MotionEvent.ACTION_UP:
                        if (mIsFlick) {
                            switch (mFlickDirection) {
                                // 下方向にフリックする
                                case Const.FLICK_PREVIOUS:
                                    shiftPreviousPage();
                                    break;
                                // 上方向にフリックする
                                case Const.FLICK_NEXT:
                                    shiftNextPage();
                                    break;
                            }
                        } else if (mScrollTo > mBaseScroll.getScrollY()) {
                            // スクロールバーが下に移動している
                            shiftPreviousPage();
                        } else if (mScrollTo < mBaseScroll.getScrollY()) {
                            // スクロールバーが上に移動している
                            shiftNextPage();
                        }
                        break;
                }
                return false;
                
              } else {
                // キャリア絵文字では、スクロールバーの制御はしない
                return false;
              }
            }
        });
    }

    /** 上ページに移動する為、メンバ変数を更新する. */
    private void shiftPreviousPage() {
        if (Const.DEBUG) {
            Log.d(Const.TAG, "shiftPreviousPage() start.");
        }

        // 先頭ページは前ページに移動しない
        if (mCurrentCategoryDataPage != Const.CATEGORY_DATA_FIRST_PAGE) {
            mCurrentCategoryDataPage--;

            // 移動開始位置を調節する
            if (mCurrentCategoryDataPage == Const.CATEGORY_DATA_FIRST_PAGE) {
                mScrollFrom = mBaseScroll.getScrollY();
                mScrollTo = 0;
            } else {
                mScrollFrom = mBaseScroll.getScrollY()
                        + mListViewManager.getDecoEmojiLinearHeight();
                mScrollTo = mListViewManager.getDecoEmojiLinearHeight();
            }

            toPreviousScroll();
        }

        if (Const.DEBUG) {
            Log.d(Const.TAG, "shiftPreviousPage() end.");
        }
    }

    /** 下ページに移動する為、メンバ変数を更新する. */
    private void shiftNextPage() {
        if (Const.DEBUG) {
            Log.d(Const.TAG, "shiftNextPage() start.");
        }

        int currentMaxPageSize;
        if (mIsExistInputHistory && mCurrentCategoryDataIndex == 0) {
            // 入力履歴
            currentMaxPageSize = mMaxPage;
        } else {
            // カテゴリ情報
            currentMaxPageSize = mMaxPage;

        }

        // 最終ページは次ページに移動しない
        if ((mCurrentCategoryDataPage + 1) != currentMaxPageSize) {
            mCurrentCategoryDataPage++;

            // 移動開始位置を調節する
            if ((mCurrentCategoryDataPage + 1) != currentMaxPageSize) {
                mScrollFrom = mBaseScroll.getScrollY()
                - mListViewManager.getDecoEmojiLinearHeight();
            } else {
                mScrollFrom = mBaseScroll.getScrollY();
            }

            mScrollTo = mListViewManager.getDecoEmojiLinearHeight();

            toNextScroll();
        }

        if (Const.DEBUG) {
            Log.d(Const.TAG, "shiftNextPage() end.");
        }
    }

    /**
     * デコ／キャリア絵文字切替テキストのonClickイベントの設定を行う.
     */
    private final OnClickListener mSwitchEmojiTabOnClick = new View.OnClickListener() {
        /**
         * デコ／キャリア絵文字切替テキスト押下時のイベント
         */
        public void onClick(View v) {
            if (v instanceof TextView) {
                TextView textView = (TextView) v;
                switch (textView.getId()) {
                    // デコ絵文字タブ押下
                    case (R.id.switch_emoji_tab_deco_text):
                        if (mCurrentEmojiValue == Const.SWITCH_VALUE_CARRIER) {
                            mSwitchEmojiTabDeco
                                    .setBackgroundResource(R.drawable.switch_emoji_tab);
                            mSwitchEmojiTabDeco.setTextColor(Color.WHITE);
                            mSwitchEmojiTabCarrier
                                    .setBackgroundResource(R.drawable.switch_emoji_tab_no_select);
                            mSwitchEmojiTabCarrier.setTextColor(Color.GRAY);
                            mCurrentEmojiValue = Const.SWITCH_VALUE_DECO;

                            // スクロールバー(Y方向)を不可視にする
                            mBaseScroll.setVerticalScrollBarEnabled(false);
                            // デコ絵文字を表示する
                            initCategoryDataList();
                        }
                        break;
                    // キャリア絵文字タブ押下
                    case (R.id.switch_emoji_tab_carrier_text):
                        if (mCurrentEmojiValue == Const.SWITCH_VALUE_DECO) {
                            mSwitchEmojiTabDeco
                                    .setBackgroundResource(R.drawable.switch_emoji_tab_no_select);
                            mSwitchEmojiTabDeco.setTextColor(Color.GRAY);
                            mSwitchEmojiTabCarrier
                                    .setBackgroundResource(R.drawable.switch_emoji_tab);
                            mSwitchEmojiTabCarrier.setTextColor(Color.WHITE);
                            mCurrentCategoryDataIndexTemp = 0;
                            mCurrentCarrierCategoryDataIndexTemp = 0;
                            mCurrentEmojiValue = Const.SWITCH_VALUE_CARRIER;

                            // スクロールバー(Y方向)を可視にする
                            mBaseScroll.setVerticalScrollBarEnabled(true);

                            clearDecoComponents();
                            initCarrierCategoryDataList();
                        }
                        break;
                }
            }
        }
    };

    /**
     * カテゴリ内ページ切替(上)ボタンのイベント設定を行う.
     */
    private void setReadMoreButtonBackEvent() {

        mReadMoreButtonBack.setOnClickListener(new View.OnClickListener() {
            /**
             * カテゴリ内ページ切替(上)ボタン押下時のイベント.
             */
            public void onClick(View v) {
                
                if (mCurrentEmojiValue == Const.SWITCH_VALUE_DECO) {
                    // デコ絵文字
                    
                    // カテゴリ情報一覧が無い時、処理をしない
                    if (mCategoryInfoArray == null) {
                        return;
                    }

                    // 先頭ページである時は最終ページを表示する
                    int currentMaxPageSize;
                    if (mIsExistInputHistory && mCurrentCategoryDataIndex == 0) {
                        // 入力履歴
                        currentMaxPageSize = mMaxPage;
                    } else {
                        // カテゴリ情報
                        currentMaxPageSize = mMaxPage;
                    }

                    // 1ページしか無い時、処理しない
                    if (currentMaxPageSize == 1) {
                        return;
                    }

                    if (mCurrentCategoryDataPage == Const.CATEGORY_DATA_FIRST_PAGE) {
                        // 1ページの場合はなにもなしない。
                        return;
                    } else {
                        // 前ページを設定する
                        mCurrentCategoryDataPage--;
                        mScrollFrom = mBaseScroll.getScrollY()
                                + mListViewManager.getDecoEmojiLinearHeight();
                        mScrollTo = mListViewManager.getDecoEmojiLinearHeight();
                        if (mCurrentCategoryDataPage == Const.CATEGORY_DATA_FIRST_PAGE) {
                            mScrollFrom = mListViewManager.getDecoEmojiLinearHeight();
                            mScrollTo = 0;
                        }
                        toPreviousScroll();
                    }
                    
                } else {
                    // キャリア絵文字
                   if (mBaseScroll.getScrollY() == 0) {
                       // 先頭の場合には何もしない。
                 	   return;
                   }
                    if (!mBaseScroll.pageScroll(ScrollView.FOCUS_UP)) {
                        mBaseScroll.scrollTo(0, mBaseScroll.getChildAt(0).getHeight());
                    }
                    
                }
                
            }
        });

        mReadMoreButtonBack.setOnLongClickListener(new View.OnLongClickListener() {
            /**
             * カテゴリ内ページ切替(上)ボタン長押し時のイベント.
             */
            public boolean onLongClick(View v) {
                
                if (mCurrentEmojiValue == Const.SWITCH_VALUE_DECO) {
                    
                    // カテゴリ情報一覧が無い時、処理をしない
                    if (mCategoryInfoArray == null) {
                        return true;
                    }
    
                    int currentMaxPageSize = -1;
    
                    if (mIsExistInputHistory && mCurrentCategoryDataIndex == 0) {
                        // 入力履歴
                        currentMaxPageSize = mMaxPage;
                    } else {
                        // カテゴリ情報
                        currentMaxPageSize = mMaxPage;
    
                    }
    
                    // 1ページしか無い時、処理しない
                    if (currentMaxPageSize == 1) {
                        return true;
                    }
    
                    // 先頭ページでなければ
                    if (mCurrentCategoryDataPage != Const.CATEGORY_DATA_FIRST_PAGE) {
                        // 先頭ページを設定する
                        mScrollFrom = 0;
                        mScrollTo = 0;
                        mCurrentCategoryDataPage = Const.CATEGORY_DATA_FIRST_PAGE;
                    }
    
                    // mCurrentCategoryDataPageを元にデコ絵文字一覧を表示する
                    toCurrentPage();
                
                } else {
                    // キャリア絵文字
                    if (mBaseScroll.getScrollY() == 0) {
                 	   // 先頭の場合には何もしない。
                        return true;
                    }
                    mBaseScroll.fullScroll(ScrollView.FOCUS_UP);
                }

                return true;
            }
        });
    }

    /**
     * カテゴリ内ページ切替(下)ボタンのイベント設定を行う.
     */
    private void setReadMoreButtonNextEvent() {

        mReadMoreButtonNext.setOnClickListener(new View.OnClickListener() {
            /**
             * カテゴリ内ページ切替(下)ボタン押下時のイベント.
             */
            public void onClick(View v) {
                
                if (mCurrentEmojiValue == Const.SWITCH_VALUE_DECO) {
                     // デコ絵文字
                    
                    // カテゴリ情報一覧が無い時、処理をしない
                    if (mCategoryInfoArray == null) {
                        return;
                    }

                    // 最終ページである時は先頭ページを表示する
                    int currentMaxPageSize;
                    if (mIsExistInputHistory && mCurrentCategoryDataIndex == 0) {
                        // 入力履歴
                        currentMaxPageSize = mMaxPage;
                    } else {
                        // カテゴリ情報
                        currentMaxPageSize = mMaxPage;
                    }

                    // 1ページしか無い時、処理しない
                    if (currentMaxPageSize == 1) {
                        return;
                    }

                    if ((mCurrentCategoryDataPage + 1) == currentMaxPageSize) {
                        // 最終ページの場合はなにもしない。
                    } else {
                        // 次ページを設定する
                        mCurrentCategoryDataPage++;
                        mScrollFrom = mBaseScroll.getScrollY()
                                - mListViewManager.getDecoEmojiLinearHeight();
                        mScrollTo = mListViewManager.getDecoEmojiLinearHeight();
                        toNextScroll();
                    }
                } else {
                    // キャリア絵文字
                    int scrollY = mBaseScroll.getScrollY();
                    int height = mBaseScroll.getHeight();
                    int count = mBaseScroll.getChildCount();
                    if (count > 0) {
                        View view = mBaseScroll.getChildAt(count - 1);
                        int top = scrollY + height;
                        int bottom = view.getBottom();
                        if (top >= bottom) {
                      	   // 末尾の場合には何もしない。
                            return;
                        }
                    }
                    if (!mBaseScroll.pageScroll(ScrollView.FOCUS_DOWN)) {
                        mBaseScroll.scrollTo(0, 0);
                    }
                    
                }
            }
        });

        mReadMoreButtonNext.setOnLongClickListener(new View.OnLongClickListener() {
            /**
             * カテゴリ内ページ切替(下)ボタン長押し時のイベント.
             */
            public boolean onLongClick(View v) {
                
                if (mCurrentEmojiValue == Const.SWITCH_VALUE_DECO) {
                    // カテゴリ情報一覧が無い時、処理をしない
                    if (mCategoryInfoArray == null) {
                        return true;
                    }
    
                    int currentMaxPageSize = -1;
    
                    if (mIsExistInputHistory && mCurrentCategoryDataIndex == 0) {
                        // 入力履歴
                        currentMaxPageSize = mMaxPage;
                    } else {
                        // カテゴリ情報
                        currentMaxPageSize = mMaxPage;
                    }
    
                    // 1ページしか無い時、処理しない
                    if (currentMaxPageSize == 1) {
                        return true;
                    }
    
                    // 最終ページでなければ
                    if ((mCurrentCategoryDataPage + 1) != currentMaxPageSize) {
                        // 最終ページを設定する
                        mScrollFrom = mListViewManager.getDecoEmojiLinearHeight();
                        mScrollTo = mListViewManager.getDecoEmojiLinearHeight();
                        mCurrentCategoryDataPage = currentMaxPageSize - 1;
                    }
    
                    // mCurrentCategoryDataPageを元にデコ絵文字一覧を表示する
                    toCurrentPage();
                    
                } else {
                    // キャリア絵文字
                    int scrollY = mBaseScroll.getScrollY();
                    int height = mBaseScroll.getHeight();
                    int count = mBaseScroll.getChildCount();
                    if (count > 0) {
                        View view = mBaseScroll.getChildAt(count - 1);
                        int top = scrollY + height;
                        int bottom = view.getBottom();
                        if (top >= bottom) {
                       	   // 末尾の場合には何もしない。
                            return true;
                        }
                    }
                     mBaseScroll.fullScroll(ScrollView.FOCUS_DOWN);
                }

                return true;
            }
        });
    }

    /**
     * バックスペースボタンの設定を行う.
     */
    private void setBackSpaceButtonEvent() {

        mBackSpaceButton.setOnClickListener(new View.OnClickListener() {
            /**
             * バックスペースボタン押下時のイベント.
             */
            public void onClick(View v) {
                
                ArrayList<DecoEmojiListResultInfo> returnValueDummy = new ArrayList<DecoEmojiListResultInfo>();
                mPreviewText.setText("");
                DecoEmojiListResultInfo decoEmojiListResultInfo;

                // 最後の文字は使用しない
                for (int i = 0; i < (mReturnValue.size() - 1); i++) {
                    decoEmojiListResultInfo = mReturnValue.get(i);
                    
                    CharSequence charSequence;
                    switch (decoEmojiListResultInfo.getTag()) {
                    case Const.SWITCH_VALUE_DECO:
                        // デコ絵文字
                        
                        charSequence = mEmojiAssist.getDecoEmojiText(Const.PREFIX_URI
                                + decoEmojiListResultInfo.getUri(), getApplicationContext());
                        break;

                    case Const.SWITCH_VALUE_CARRIER:
                        // キャリア絵文字
                        
                        charSequence = decoEmojiListResultInfo.getCode();
                        
                        break;
                        
                    default:
                        
                        throw new IllegalArgumentException("Unsupported tag " + decoEmojiListResultInfo.getTag());
                        
                    }
                    
                    mPreviewText.append(charSequence);
                    returnValueDummy.add(decoEmojiListResultInfo);
                }
                mReturnValue = returnValueDummy;

            }
        });

    }

    /**
     * カテゴリ名称の設定を行う.
     */
    private void setCategoryName() {
        ArrayAdapter<String> adapter = new ArrayAdapter<String>(getApplicationContext(),
                android.R.layout.simple_spinner_item);
        adapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);

        if (mCurrentEmojiValue == Const.SWITCH_VALUE_DECO) {
            // デコ絵文字

            // カテゴリ名称一覧を追加する
            if (mCategoryInfoArray != null) {
                for (int i = 0; i < mCategoryInfoArray.length; i++) {
                    if (mIsExistInputHistory && i == 0) {
                        // 1件目は入力履歴から設定する
                        adapter.add(getResources().getString(R.string.input_history_name));
                    } else {
                        // カテゴリ情報から設定する
                        if (mIsLanguageJapan) {
                            adapter.add(mCategoryInfoArray[i].getCategoryName_jpn());
                        } else {
                            adapter.add(mCategoryInfoArray[i].getCategoryName_eng());
                        }
                    }
                }
            } else {
                // リトライ処理終了済み
                if (mNoPictDData == NO_PICT_D_DATA_FLAG) {
                    adapter.add(getString(R.string.pict_d_result_message));
                }

                // リトライ処理中
                if (mRetryCount == MAX_RETRY_COUNT && mNoPictDData != NO_PICT_D_DATA_FLAG) {
                    adapter.add(getString(R.string.pict_d_reading_message));
                }
                else if (mRetryCount == (MAX_RETRY_COUNT - 1)) {
                    adapter.add(getString(R.string.pict_d_reading_message)
                            + getString(R.string.pict_d_reading_message_sub));
                }
                else if (mRetryCount == (MAX_RETRY_COUNT - 2)) {
                    adapter.add(getString(R.string.pict_d_reading_message)
                            + getString(R.string.pict_d_reading_message_sub)
                            + getString(R.string.pict_d_reading_message_sub));
                }
                else if (mRetryCount == (MAX_RETRY_COUNT - 3)) {
                    adapter.add(getString(R.string.pict_d_reading_message)
                            + getString(R.string.pict_d_reading_message_sub)
                            + getString(R.string.pict_d_reading_message_sub)
                            + getString(R.string.pict_d_reading_message_sub));
                }
                else if (mRetryCount == (MAX_RETRY_COUNT - 4)) {
                    adapter.add(getString(R.string.pict_d_reading_message)
                            + getString(R.string.pict_d_reading_message_sub)
                            + getString(R.string.pict_d_reading_message_sub)
                            + getString(R.string.pict_d_reading_message_sub)
                            + getString(R.string.pict_d_reading_message_sub));
                }
                else if (mRetryCount == (MAX_RETRY_COUNT - 5)) {
                    adapter.add(getString(R.string.pict_d_reading_message)
                            + getString(R.string.pict_d_reading_message_sub)
                            + getString(R.string.pict_d_reading_message_sub)
                            + getString(R.string.pict_d_reading_message_sub)
                            + getString(R.string.pict_d_reading_message_sub)
                            + getString(R.string.pict_d_reading_message_sub));
                }
                else if (mRetryCount == 0) {
                    adapter.add(getString(R.string.pict_d_result_message));
                    adapter.remove(getString(R.string.pict_d_reading_message_sub));
                }
            }

            // カテゴリ名称一覧を設定します
            mCategoryName.setAdapter(adapter);
            mCategoryName.setSelection(mCurrentCategoryDataIndex);
            mCategoryName.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
                /**
                 * カテゴリ名称が選択された時に呼び出されるイベント.
                 */
                public void onItemSelected(AdapterView<?> parent, View view, int position,
                        long id) {
                    if (mCurrentCategoryDataIndex != position) {
                        mMaxPage = 0;
                        mCurrentCategoryDataIndex = position;
                        mCurrentCategoryDataPage = Const.CATEGORY_DATA_FIRST_PAGE;
                        mScrollFrom = 0;
                        mScrollTo = 0;

                        toCurrentPage();
                    }
                }

                /**
                 * 何も選択されなかった時に呼び出されるイベント.
                 */
                public void onNothingSelected(AdapterView<?> parent) {
                }
            });

        } else {
            // キャリア絵文字

            // カテゴリ名称一覧を追加する
            if (mCarrierEmojiCategoryInfoList != null) {
                
                for (CarrierEmojiCategoryInfo info : mCarrierEmojiCategoryInfoList) {
                     adapter.add(info.getCategoryName());
                }
                
            } else {
                adapter.add("");
            }

            // カテゴリ名称一覧を設定します
            mCategoryName.setAdapter(adapter);
            mCategoryName.setSelection(mCurrentCarrierCategoryDataIndex);
            mCategoryName.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
                /**
                 * カテゴリ名称が選択された時に呼び出されるイベント.
                 */
                public void onItemSelected(AdapterView<?> parent, View view, int position,
                        long id) {
                    if (mCurrentCarrierCategoryDataIndex != position) {
                        mCurrentCarrierCategoryDataIndex = position;
                        mScrollFrom = 0;
                        mScrollTo = 0;

                        showCarrierEmoji(mCurrentCarrierCategoryDataIndex);
                    }
                }

                /**
                 * 何も選択されなかった時に呼び出されるイベント.
                 */
                public void onNothingSelected(AdapterView<?> parent) {
                }
            });
        }
    }

    /**
     * 指定したカテゴリに紐付くキャリア絵文字一覧を表示する
     * @param categoryIndex
     */
    private void showCarrierEmoji(int categoryIndex) {

        mDecoEmojiListBaseLinear.removeAllViews();

        // EmojiAssistに登録されたTextViewをクリアする
        mEmojiAssist.clearView();
        mAddViewList.clear();

        // カテゴリ情報一覧が無い時、処理をしない
        if ( (mCarrierEmojiCategoryInfoList == null)
                || (mCarrierEmojiCategoryInfoList.isEmpty()) ) {
            return;
        }

        // キャリア絵文字一覧を表示するレイアウトを設定する
        LinearLayout targetLinear = new LinearLayout(getApplicationContext());
        targetLinear.setVisibility(View.GONE);
        targetLinear.setOrientation(LinearLayout.VERTICAL);
        targetLinear.setMinimumHeight(mListViewManager.getDecoEmojiLinearHeight());
        targetLinear.removeAllViews();

        // 入力履歴に含まれる絵文字コードを取得する
        if (mIsExistCarrierInputHistory && categoryIndex == 0) {
            List<String> carrierInputHistories = mInputHistoryDao.loadHistoriesCarrier();
            mCarrierEmojiCategoryInfoList.get(categoryIndex).setmEmojiCodeList(carrierInputHistories);
        }
        
        // 選択されたカテゴリの絵文字コードを取得
        CarrierEmojiCategoryInfo categoryInfo = mCarrierEmojiCategoryInfoList.get(categoryIndex);
        List<String> emojiList = categoryInfo.getmEmojiCodeList();
        
        if (emojiList != null && emojiList.size() != 0) {

            if (Const.DEBUG) {
                Log.d(Const.TAG, CLASS_NAME
                        + "showCategoryList() mCurrentDecoEmojiUriList.size["
                        + mCurrentDecoEmojiUriList.size() + "]");
            }

            int decoEmojiCounter = 0;
            int decoEmojiWidth;
            int decoEmojiHeight;
            TextView decoEmojiView;
            // 全体のデコ絵文字を表示する(行)
            while (decoEmojiCounter < emojiList.size()) {

                mDecoEmojiListInner = new LinearLayout(getApplicationContext());
                mDecoEmojiListInner.setOrientation(LinearLayout.HORIZONTAL);
                
                // 1行分のキャリア絵文字を表示する(列)
                for (int j = 0; j < mListViewManager.getDecoEmojiMaxCols()
                        && decoEmojiCounter < emojiList.size(); j++) {
                    decoEmojiView = new TextView(getApplicationContext());
                    // 1行の最終キャリア絵文字は横幅を変更する
                    if ((j + 1) == mListViewManager.getDecoEmojiMaxCols()) {
                        decoEmojiWidth = mListViewManager.getDecoEmojiWidthEndOfLine();
                    } else {
                        decoEmojiWidth = mListViewManager.getDecoEmojiWidth();
                    }

                    decoEmojiHeight = mListViewManager.getDecoEmojiHeight();
                    
                    decoEmojiView.setBackgroundResource(R.drawable.buttons_background_type1);
                    decoEmojiView.setLayoutParams(new GridView.LayoutParams(decoEmojiWidth, decoEmojiHeight));
                    decoEmojiView.setPadding(0, 0, 0, 0);
                    decoEmojiView.setGravity(Gravity.CENTER_HORIZONTAL | Gravity.CENTER_VERTICAL);
                    decoEmojiView.setOnClickListener(mCarrierEmojiOnClick);
                    decoEmojiView.setTag(emojiList.get(decoEmojiCounter).toString());

                    decoEmojiView.setText(emojiList.get(decoEmojiCounter).toString());

                    // 一覧表示される絵文字サイズを大きくする
                    decoEmojiView.setTextSize(DECOEMOJI_SIZE);

                    addView(decoEmojiView);
                   
                    mDecoEmojiListInner.addView(decoEmojiView);
                    decoEmojiCounter++;
                }
                targetLinear.addView(mDecoEmojiListInner);
            }
            targetLinear.setVisibility(View.VISIBLE);
        }

        mDecoEmojiListBaseLinear.addView(targetLinear);
        
        setPreviewText();
        
        startAnimation();


        mBaseScroll.scrollTo(0, mScrollFrom);
        mBaseScroll.smoothScrollTo(0, mScrollTo);
    }
    
    /**
     * mCurrentCategoryDataPageを元にデコ絵文字一覧を表示する.
     */
    private void toCurrentPage() {
        mDecoEmojiListBaseLinear.removeAllViews();

        // EmojiAssistに登録されたTextViewをクリアする
        mEmojiAssist.clearView();
        mAddViewList.clear();

        showCategoryList(mCurrentCategoryDataPage - 1);
        showCategoryList(mCurrentCategoryDataPage);
        showCategoryList(mCurrentCategoryDataPage + 1);

        setPreviewText();

        startAnimation();

        mBaseScroll.scrollTo(0, mScrollFrom);
        mBaseScroll.smoothScrollTo(0, mScrollTo);
    }

    /**
     * 上ページに遷移する処理を行う.
     */
    private void toPreviousScroll() {
        LinearLayout linear1st = (LinearLayout) mDecoEmojiListBaseLinear.getChildAt(0);
        LinearLayout linear2nd = (LinearLayout) mDecoEmojiListBaseLinear.getChildAt(1);
        mDecoEmojiListBaseLinear.removeAllViews();

        // EmojiAssistに登録されたTextViewをクリアする
        mEmojiAssist.clearView();
        mAddViewList.clear();

        showCategoryList(mCurrentCategoryDataPage - 1);
        mDecoEmojiListBaseLinear.addView(linear1st);
        mDecoEmojiListBaseLinear.addView(linear2nd);

        setDecoEmojiViewToEmojiAssist(linear1st);
        setDecoEmojiViewToEmojiAssist(linear2nd);

        setPreviewText();

        startAnimation();

        mBaseScroll.scrollTo(0, mScrollFrom);
        mBaseScroll.smoothScrollTo(0, mScrollTo);
    }

    /**
     * 下ページに遷移する処理を行う.
     */
    private void toNextScroll() {
        LinearLayout linear2nd = (LinearLayout) mDecoEmojiListBaseLinear.getChildAt(1);
        LinearLayout linear3rd = (LinearLayout) mDecoEmojiListBaseLinear.getChildAt(2);
        mDecoEmojiListBaseLinear.removeAllViews();

        // EmojiAssistに登録されたTextViewをクリアする
        mEmojiAssist.clearView();
        mAddViewList.clear();

        mDecoEmojiListBaseLinear.addView(linear2nd);
        mDecoEmojiListBaseLinear.addView(linear3rd);
        showCategoryList(mCurrentCategoryDataPage + 1);

        setDecoEmojiViewToEmojiAssist(linear2nd);
        setDecoEmojiViewToEmojiAssist(linear3rd);

        setPreviewText();

        startAnimation();

        mBaseScroll.scrollTo(0, mScrollFrom);
        mBaseScroll.smoothScrollTo(0, mScrollTo);
    }

    /**
     * レイアウトが持つデコ絵文字一覧をEmojiAssistに登録する.
     *
     * @param target デコ絵文字一覧を持つレイアウト
     */
    private void setDecoEmojiViewToEmojiAssist(LinearLayout rows) {
        for (int i = 0; i < rows.getChildCount(); i++) {
            LinearLayout cols = (LinearLayout) rows.getChildAt(i);
            for (int j = 0; j < cols.getChildCount(); j++) {
                addView((TextView) cols.getChildAt(j));
            }
        }
    }

    /**
     * デコ絵文字一覧に表示させるTextViewの設定を行う.
     *
     * @param textView
     *            デコ絵文字を表示させるTextView
     * @param position
     *            デコ絵文字一覧の要素位置(index)
     * @param width
     *            TextViewの横幅
     * @param height
     *            TextViewの縦幅
     * @return パラメータを設定したTextView
     */
    private TextView getCategoryTextView(TextView textView, int position, int width, int height, CharSequence charSequence) {
        textView.setBackgroundResource(R.drawable.buttons_background_type1);
        textView.setLayoutParams(new GridView.LayoutParams(width, height));
        textView.setPadding(0, 0, 0, 0);
        textView.setGravity(Gravity.CENTER_HORIZONTAL | Gravity.CENTER_VERTICAL);
        textView.setSingleLine();
        textView.setOnClickListener(mDecoEmojiOnClick);
        textView.setTag(mCurrentDecoEmojiUriList.get(position).toString());

        textView.setText(charSequence);

        // 一覧表示される絵文字サイズを大きくする
        textView.setTextSize(DECOEMOJI_SIZE);

        addView(textView);

        return textView;
    }

    /**
     * デコ絵文字一覧を表示するTextViewのonClickイベントの設定を行う.
     */
    private final OnClickListener mDecoEmojiOnClick = new OnClickListener() {
        /**
         * デコ絵文字を表示しているTextView押下時のイベント.
         */
        public void onClick(View v) {
            if (mReturnValue.size() < Const.MAX_PREVIEW_TEXT) {
                DecoEmojiListResultInfo decoEmojiListResultInfo = new DecoEmojiListResultInfo();
                TextView textView = (TextView) v;
                decoEmojiListResultInfo.setUri((String) textView.getTag());
                decoEmojiListResultInfo.setTag(Const.SWITCH_VALUE_DECO);
                mReturnValue.add(decoEmojiListResultInfo);

                if (Const.DEBUG) {
                    Log.d(Const.TAG, "mDecoEmojiOnClick.onClick() Selected DecoEmoji. uri["
                            + (String) textView.getTag() + "]");
                }

                // プレビューテキストに表示する
                if (mCurrentInputType == INPUT_TYPE_SINGLE) {
                    setResultData();
                }
                else {
                    // プレビューテキストに表示する
                    setPreviewText();
                }
            }
        }
    };

    /**
     * キャリア絵文字一覧を表示するTextViewのonClickイベントの設定を行う.
     */
    private OnClickListener mCarrierEmojiOnClick = new OnClickListener() {
        
        /**
         * デコ絵文字を表示しているTextView押下時のイベント.
         */
        public void onClick(View v) {
            if (mReturnValue.size() < Const.MAX_PREVIEW_TEXT) {
                DecoEmojiListResultInfo decoEmojiListResultInfo = new DecoEmojiListResultInfo();
                TextView textView = (TextView) v;
                decoEmojiListResultInfo.setCode((String) textView.getTag());
                decoEmojiListResultInfo.setTag(Const.SWITCH_VALUE_CARRIER);
                mReturnValue.add(decoEmojiListResultInfo);

                if (Const.DEBUG) {
                    Log.d(Const.TAG, "mCarrierEmojiOnClick.onClick() Selected CarrierEmoji. code["
                            + (String) textView.getTag() + "]");
                }

                // プレビューテキストに表示する
                if (mCurrentInputType == INPUT_TYPE_SINGLE) {
                    setResultData();
                }
                else {
                    // プレビューテキストに表示する
                    setPreviewText();
                }
            }
        }
    };

    /**
     * 表示しているデコ絵文字をクリアする.
     */
    private void clearDecoComponents() {
        // カテゴリ名称をクリアする
        ArrayAdapter<String> adapter = new ArrayAdapter<String>(getApplicationContext(),
                android.R.layout.simple_spinner_item);
        adapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        adapter.add("");
        mCategoryName.setAdapter(adapter);

        // デコ絵文字一覧をクリアする
        mDecoEmojiListBaseLinear.removeAllViews();
        mCategoryInfoArray = null;
    }

    /**
     * Activityをfinish()する. 引数に応じて値を返却する。
     *
     * @param isOnClickEnter
     *            true:確定ボタン押下後/false:確定ボタン押下後以外
     */
    private void sendResultData(boolean isOnClickEnter) {
        if (Const.DEBUG) {
            Log.d(Const.TAG, CLASS_NAME + "sendResultData() start. isOnClickEnter["
                    + isOnClickEnter + "] isSendFinish[" + isSendFinish + "]");
        }

        // 入力履歴テーブル操作用DAOクラスを閉じる
        if (mInputHistoryDao != null) {
            mInputHistoryDao.closeHistories();
        }

        // 確定ボタン押下時のみ、Extraを送信する
        if (isOnClickEnter) {
            Intent data = (Intent) getIntent();
            data.putParcelableArrayListExtra(Const.EXTRA_RETURN_VALUE, mReturnValue);
            setResult(DecoEmojiList.RESULT_OK, data);
        }

        // finish送信済みの時、finishを行わない
        if (!isSendFinish) {
            finish();
            isSendFinish = true;
        }

        // 遷移アニメを指定する
        overridePendingTransition(R.anim.decoemojilist_close_enter,
                R.anim.decoemojilist_close_exit);

        if (Const.DEBUG) {
            Log.d(Const.TAG, CLASS_NAME + "sendResultData() end.");
        }
    }

    private void setResultData() {
        // 入力履歴をデータベースに保存する
        if (mReturnValue.size() != 0) {
            DecoEmojiListResultInfo decoEmojiListResultInfo;
            for (int i = 0; i < mReturnValue.size(); i++) {
                decoEmojiListResultInfo = mReturnValue.get(i);
                switch (decoEmojiListResultInfo.getTag()) {
                // デコ絵文字
                case Const.SWITCH_VALUE_DECO:
                    try {
                        int rc = mIDecoEmojiManagerIF.aidl_changeHistory(decoEmojiListResultInfo.getUri());
                    } catch (RemoteException e) {
                        break;
                    }
                    break;
                // キャリア絵文字
                case Const.SWITCH_VALUE_CARRIER:
                    mInputHistoryDao.updateHistoryCarrier(decoEmojiListResultInfo.getCode());
                    break;
                default:
                    throw new IllegalArgumentException("Unsupported tag " + decoEmojiListResultInfo.getTag());
                }
            }
        }
        // 結果を返却する
        sendResultData(true);
    }

    /**
     * 一覧表示の行途中で改行が入る場合、最後の絵文字の背景を行の最後まで伸ばす。
     *
     * @param viewCnt 現在処理中の絵文字のViewTextインデックス
     * @param decoEmojiCounter 現在処理中の絵文字のURIリストインデックス
     * @param position 現在処理中の絵文字の行における位置
     * @param height 行の高さ
     */
    private void updatePreViewLayOut(int viewCnt, int decoEmojiCounter, int position, int height) {
        int resizeWidth = (mListViewManager.getDecoEmojiMaxCols() - position) * mListViewManager.getDecoEmojiWidth() + (mListViewManager.getDisplayWidth() % mListViewManager.getDecoEmojiMaxCols());
        TextView preView = (TextView) mDecoEmojiListInner.getChildAt(viewCnt - 1);
        mEmojiAssist.removeView(preView);
        mAddViewList.remove(preView);
        mDecoEmojiListInner.removeViewAt(viewCnt - 1);
        CharSequence charSequence = mEmojiAssist.getDecoEmojiText(Const.PREFIX_URI
                + mCurrentDecoEmojiUriList.get(decoEmojiCounter - 1).toString(), getApplicationContext());
        preView = getCategoryTextView(preView, decoEmojiCounter - 1, resizeWidth, height, charSequence);
        mDecoEmojiListInner.addView(preView);
    }

    /**
     * ページ情報保持クラス 
     *
     */
    final class PageInfo {
    	
    	/** デコ絵文字表示件数 */
    	private int mEmojiCount;
    	/** デコ絵文字取得開始位置 */
    	private int mEmojiStart;
    	/** デコメピクチャ表示件数 */
    	private int mPictureCount;
    	/** デコメピクチャ開始位置 */
    	private int mPictureStart;
    	
    	/**
    	 * コンストラクタ
    	 * @param emojiCount デコ絵文字表示件数 
    	 * @param emojiStart デコ絵文字取得開始位置
    	 * @param pictureCount デコメピクチャ表示件数
    	 * @param pictureStart デコメピクチャ開始位置
    	 */
    	public PageInfo(int emojiCount, int emojiStart, int pictureCount, int pictureStart){
    		mEmojiCount = emojiCount;
    		mEmojiStart = emojiStart;
    		mPictureCount = pictureCount;
    		mPictureStart = pictureStart;
    	}
    	
    	/**
    	 * デコ絵文字表示件数を設定する
    	 * @param count　デコ絵文字表示件数
    	 */
    	public void setEmojiCount(int count) {
    		mEmojiCount = count;
    	}
    	
    	/**
    	 * デコ絵文字表示件数を取得する
    	 * @return デコ絵文字表示件数
    	 */
    	public int getEmojiCount() {
    		return mEmojiCount;
    	}
    	
    	/**
    	 * デコ絵文字取得開始位置を設定する
    	 * @param start
    	 */
    	public void setEmojiStart(int start) {
    		mEmojiStart = start;
    	}
    	
    	/**
    	 * デコ絵文字取得開始位置を取得する
    	 * @return
    	 */
    	public int getEmojiStart() {
    		return mEmojiStart;
    	}

    	/**
    	 * デコメピクチャ表示件数を設定する
    	 * @param count　デコメピクチャ表示件数
    	 */
    	public void setPictureCount(int count) {
    		mPictureCount = count;
    	}
    	
    	/**
    	 * デコメピクチャ表示件数を取得する
    	 * @return デコメピクチャ表示件数
    	 */
    	public int getPictureCount() {
    		return mPictureCount;
    	}
    	
    	/**
    	 * デコメピクチャ開始位置を設定する
    	 * @param start デコメピクチャ開始位置
    	 */
    	public void setPictureStart(int start) {
    		mPictureStart = start;
    	}
    	
    	/**
    	 * デコメピクチャ開始位置を取得する。
    	 * @return デコメピクチャ開始位置
    	 */
    	public int getPictureStart() {
    		return mPictureStart;
    	}
    	
    }

    /**
     * リストにデコレーション絵文字が含まれている場合には、アニメーションの開始を行う.
     */
    private void startAnimation() {
        Boolean isDeco = false;
        if (mAddViewList != null && mAddViewList.size() > 0) {
            for (int i = 0; mAddViewList.size() > i ; i++) {
                Spannable text = new SpannableString(mAddViewList.get(i).getText());
                int ret = mEmojiAssist.checkTextData(text);
                if (ret > EmojiAssist.TYPE_TEXT_EMOJI) {
                    isDeco = true;
                    break;
                }
            }
            if (isDeco) {
                mEmojiAssist.startAnimation();
            } else {
                mEmojiAssist.stopAnimation();
            }
        } else {
            mEmojiAssist.stopAnimation();
        }
    }
    /**
     * 絵文字、デコレーション絵文字が含まれている場合には、Viewの追加を行う.
     * @param v
     *            追加するView
     */
    private void addView(TextView v) {
        Spannable span = new SpannableString(v.getText());
        int ret = mEmojiAssist.checkTextData(span);
        if (ret > EmojiAssist.TYPE_TEXT_NORMAL) {
            mEmojiAssist.addView(v);
            mAddViewList.add(v);
        }
    }
}
